@@ -553,30 +553,30 @@ static RETSIGTYPE sigterm()
 
 int pktget(struct modemchannel *chan, unsigned char *data, unsigned int len)
 {
-	unsigned int i, j;
+	unsigned int i, j, n = len;
 
 	i = (chan->pkt.htx.rd - chan->pkt.htx.wr - 1) % TXBUFFER_SIZE;
 	if (i > KISSINBUF_SIZE*6/5)
 		kiss_input(chan);
 	if (chan->pkt.inhibittx || chan->pkt.htx.rd == chan->pkt.htx.wr)
-		return -1;
-	while (len > 0) {
+		return 0;
+	while (n > 0) {
 		if (chan->pkt.htx.wr >= chan->pkt.htx.rd)
 			j = chan->pkt.htx.wr - chan->pkt.htx.rd;
 		else
 			j = TXBUFFER_SIZE - chan->pkt.htx.rd;
-		if (j > len)
-			j = len;
+		if (j > n)
+			j = n;
 		if (!j)
 			break;
 		memcpy(data, &chan->pkt.htx.buf[chan->pkt.htx.rd], j);
 		data += j;
-		len -= j;
+		n -= j;
 		chan->pkt.htx.rd = (chan->pkt.htx.rd + j) % TXBUFFER_SIZE;
 	}
-	if (len > 0)
-		memset(data, 0, len);
-	return 0;
+	if (n > 0)
+		memset(data, 0, n);
+	return (len - n);
 }
 
 void pktput(struct modemchannel *chan, const unsigned char *data, unsigned int len)
@@ -848,18 +848,22 @@ void pktinitmkiss(struct modemchannel *chan, const char *params[])
                 logerr(MLOG_FATAL, "ioctl: SIOCSIFENCAP");
 	/* try to set the interface name */
 	if (params[0]) {
-		if (ioctl(master, SIOCGIFNAME, ifr.ifr_name) == -1)
-			logerr(MLOG_FATAL, "ioctl: SIOCGIFNAME");
+                if (ioctl(master, SIOCGIFNAME, &ifr) == -1)
+                        logerr(MLOG_FATAL, "ioctl: SIOCGIFNAME");
                 if (strcmp(params[0], ifr.ifr_name)) {
+                        if ((fd = socket(PF_INET, SOCK_DGRAM, 0)) == -1)
+                                logerr(MLOG_FATAL, "socket (setifname)");
                         strncpy(ifr.ifr_newname, params[0], sizeof(ifr.ifr_newname));
-                        if (ioctl(master, SIOCSIFNAME, &ifr) == -1) {
+                        ifr.ifr_newname[sizeof(ifr.ifr_newname) - 1] = 0;
+                        if (ioctl(fd, SIOCSIFNAME, &ifr) == -1) {
                                 logerr(1, "ioctl: SIOCSIFNAME");
                                 logprintf(1, "mkiss: cannot set ifname to %s, using %s (old kernel version?)\n",
                                           params[0], ifr.ifr_name);
                         }
+                        close(fd);
                 }
 	}
-	if (ioctl(master, SIOCGIFNAME, ifr.ifr_name) == -1)
+	if (ioctl(master, SIOCGIFNAME, &ifr) == -1)
 		logerr(MLOG_FATAL, "ioctl: SIOCGIFNAME");
 	strncpy(chan->pkt.kiss.ifname, ifr.ifr_name, sizeof(chan->pkt.kiss.ifname));
 	/* start the interface */
