@@ -3,8 +3,8 @@
 /*
  *      main.c  --  Soundmodem main.
  *
- *      Copyright (C) 1999-2000
- *        Thomas Sailer (sailer@ife.ee.ethz.ch)
+ *      Copyright (C) 1999-2001
+ *        Thomas Sailer (t.sailer@alumni.ethz.ch)
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -221,6 +221,7 @@ static int parsecfg(xmlDocPtr doc, xmlNodePtr node, struct state *state, unsigne
 	const char *par[MAXPAR];
 	struct modemchannel *chan;
         pthread_attr_t rxattr;
+	size_t stacksize;
 	unsigned int samplerate = 5000, mode;
 
 	for (; node; node = node->next) {
@@ -287,13 +288,19 @@ static int parsecfg(xmlDocPtr doc, xmlNodePtr node, struct state *state, unsigne
 		getparam(doc, audio, ioparams_soundcard, par);
 		state->audioio = ioopen_soundcard(&samplerate, mode, par);
 	}
-	if (state->audioio)
+	if (!state->audioio)
                 logprintf(MLOG_FATAL, "cannot start audio\n");
 	for (chan = state->channels; chan; chan = chan->next) {
 		if (chan->demod) {
 			chan->demod->init(chan->demodstate, samplerate, &chan->rxbitrate);
                         if (pthread_attr_init(&rxattr))
 				logerr(MLOG_FATAL, "pthread_attr_init");
+			/* needed on FreeBSD, according to driehuis@playbeing.org */
+			if (pthread_attr_getstacksize(&rxattr, &stacksize))
+				logerr(MLOG_ERROR, "pthread_attr_getstacksize");
+			else if (stacksize < 256*1024)
+				if (pthread_attr_setstacksize(&rxattr, 256*1024))
+					logerr(MLOG_ERROR, "pthread_attr_setstacksize");
 #ifdef HAVE_SCHED_H
                         if (schedrr && *schedrr) {
                                 struct sched_param schp;
@@ -317,6 +324,14 @@ static int parsecfg(xmlDocPtr doc, xmlNodePtr node, struct state *state, unsigne
 
 /* ---------------------------------------------------------------------- */
 
+#ifdef HAVE_MLOCKALL
+#define MLOCKOPT "M"
+#define MLOCKHLP " [-M]"
+#else /* HAVE_MLOCKALL */
+#define MLOCKOPT ""
+#define MLOCKHLP ""
+#endif /* HAVE_MLOCKALL */
+
 static void parseopts(int argc, char *argv[])
 {
 	static const struct option long_options[] = {
@@ -335,7 +350,7 @@ static void parseopts(int argc, char *argv[])
 	pid_t pid;
 	unsigned char uch;	
 
-        while ((c = getopt_long(argc, argv, "v:sSc:RMD", long_options, NULL)) != EOF) {
+        while ((c = getopt_long(argc, argv, "v:sSc:RD" MLOCKOPT, long_options, NULL)) != EOF) {
                 switch (c) {
                 case 'v':
                         verblevel = strtoul(optarg, NULL, 0);
@@ -357,10 +372,12 @@ static void parseopts(int argc, char *argv[])
                         schedrr = 1;
                         break;
 
+#ifdef HAVE_MLOCKALL
                 case 'M':
                         lockmem = 1;
                         break;
-                        
+#endif /* HAVE_MLOCKALL */
+
 		case 'D':
 			daemonize = 1;
 			break;
@@ -371,7 +388,7 @@ static void parseopts(int argc, char *argv[])
                 }
         }
 	if (err) {
-                fprintf(stderr, "usage: [-v <verblevel>] [-s] [-S] [-R] [-M] [-c <configname>] <configfile>\n");
+                fprintf(stderr, "usage: [-v <verblevel>] [-s] [-S] [-R]" MLOCKHLP " [-c <configname>] <configfile>\n");
                 exit(1);
         }
 	loginit(verblevel, tosyslog);
@@ -428,10 +445,12 @@ static void parseopts(int argc, char *argv[])
         /*
          * lock memory down
          */
+#ifdef HAVE_MLOCKALL
         if (lockmem) {
                 if (mlockall(MCL_CURRENT | MCL_FUTURE) == -1) 
                         logerr(MLOG_ERROR, "mlockall");
         }
+#endif /* HAVE_MLOCKALL */
 #ifdef HAVE_SCHED_H
         if (schedrr) {
                 struct sched_param schp;
