@@ -52,11 +52,17 @@
 #include "ppdev.h"
 #endif
 
+#if defined HAVE_STRING_H
+# include <string.h>
+#else
+# include <strings.h>
+#endif
+
 /* ---------------------------------------------------------------------- */
 
 struct modemparams pttparams[] = {
-	{ "file", "PTT Driver", "Path name of the serial or parallel port driver for outputting PTT", "/dev/ttyS0", MODEMPAR_COMBO, 
-	  { c: { { "/dev/ttyS0", "/dev/ttyS1", "/dev/parport0", "/dev/parport1" } } } },
+	{ "file", "PTT Driver", "Path name of the serial or parallel port driver for outputting PTT", "none", MODEMPAR_COMBO, 
+	  { c: { { "none", "/dev/ttyS0", "/dev/ttyS1", "/dev/parport0", "/dev/parport1" } } } },
 	{ NULL }
 };
 
@@ -69,10 +75,13 @@ int pttinit(struct pttio *state, const char *params[])
 	unsigned char x;
 	unsigned int y = 0;
 
-	if (!path || !path[0])
+	if (!path || !path[0] || !strcasecmp(path, "none")) {
+		state->fd = -1;
 		return 0;
+	}
 	if ((fd = open(path, O_RDWR, 0)) < 0) {
 		logprintf(MLOG_ERROR, "Cannot open PTT device \"%s\"\n", path);
+		state->fd = -1;
 		return -1;
 	}
 	if (!ioctl(fd, TIOCMBIC, &y)) {
